/**************************************************************************
 *	Copyright (C) 2004  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "fe_main.hpp"

// Move this into the thing.
GLfloat mat_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat mat_shininess[] = { 150. };
GLfloat light_position[] = { 1.0f, 1.0f, 1.0f, 0.0 };
GLfloat light_position2[] = { -1.0f, -1.0f, -1.0f, 0.0 };

GLfloat white_light[] = { 4.0f, 4.0f, 4.0f, 1.0f };

#ifdef WIN32 // Should be changed to OGL version or something more robust.
PFNGLBINDBUFFERARBPROC	pglBindBufferARB;
PFNGLGENBUFFERSARBPROC	pglGenBuffersARB;
PFNGLBUFFERDATAARBPROC	pglBufferDataARB;

PFNGLLOCKARRAYSEXTPROC		glLockArraysEXT;
PFNGLUNLOCKARRAYSEXTPROC	glUnlockArraysEXT;
#endif

void drawMenu();
Uint32 timerProc(Uint32 interval, void *param);
int rFps = 0;

FeMain::FeMain()
{
	mouseDown = FALSE;
	key_mode = 0; // 0 - normal, 1 - Z, 2 -P
	prevX = 0;
	prevY = 0;
	ss_id = 1;
	fps = 0;
	ff = 0;
	lastframe = 0;
	thisframe = 0;
	nmapScan = NULL;
	m_Config = new XMLConfig();
	m_ScanParser = new ScanParser();
	m_glUtil = new GLUtil();
	m_Config->m_glUtil = m_glUtil;
	m_Config->m_TexStore->m_glUtil = m_glUtil; // Maybe texcache should be inside glutil
}

FeMain::~FeMain()
{
	TTF_Quit();
	SDL_Quit();
	
	if(nmapScan)
		delete nmapScan;

	delete m_ScanParser;
	delete m_Config;
	delete m_glUtil;
}


/**
 *
 * \param file 
 * \return 
 */
bool FeMain::init(string file)
{

	int done = 0;

	if ( SDL_Init(SDL_INIT_VIDEO|SDL_INIT_TIMER) < 0 ) {
		fprintf(stderr, "Unable to initialize SDL: %s\n", SDL_GetError());
		exit(1);
	}

	if(TTF_Init()==-1) 
	{
		printf("TTF_Init: %s\n", TTF_GetError());
		return FALSE;
	}

	// We need this here so we can get the resolution
	m_Config->initParser();			// Initialize XML Parser
	
	if(!m_Config->loadSettings())	// Load Fe3D config file
	{
		cout<<"Failed to load settings near file: "<<file<<endl;
		return FALSE;
	}

	m_glUtil->m_FontFile = m_Config->m_FontFile;
//	cout<<"Setting font file to: "<<m_glUtil->m_FontFile<<endl; 
	cout<<"Setting Video Mode to: "<<m_Config->m_ResW<<" : "<<m_Config->m_ResH<<endl;

	screen=SDL_SetVideoMode(m_Config->m_ResW,m_Config->m_ResH,32,SDL_SWSURFACE|SDL_HWPALETTE|SDL_OPENGL);
	if ( screen == NULL ) 
	{
		fprintf(stderr, "Unable to create OpenGL screen: %s\n", SDL_GetError());
		return FALSE;
	}

	// Set the title bar in environments that support it 
#if !defined(__APPLE__) && !defined(__MACH__)
	sprintf(title, "fe3d v %s build %i", PRODUCT_VERSION, BUILD_NUM);
	SDL_WM_SetCaption(title, NULL);
#endif
	
	m_Config->loadOSInfo();			// Load Texture/Geometry descriptor

	InitGL(m_Config->m_ResW,m_Config->m_ResH);
	lastframe=SDL_GetTicks();

	nmapScan = new ScanDraw(); 
	nmapScan->m_Config = m_Config;
	nmapScan->m_TexStore = m_Config->getTextureStore();
	nmapScan->m_Dirty = TRUE;
	nmapScan->fpsOn = TRUE;
	nmapScan->rotateOn = FALSE;

	m_ScanParser->addParser(new NmapParser());
	m_ScanParser->addParser(new LanGuardParser());

	m_ScanParser->setConfig(m_Config);
	m_ScanParser->initParser();
	m_ScanParser->setScan(nmapScan);
	m_ScanParser->parseFile(file);

 	SDL_EnableKeyRepeat(100, 50);
	SDL_AddTimer(2000, timerProc, 0); // for fps counter

	while ( ! done ) 
	{
		DrawGLScene();
		done = EventHandler();
	}

	return TRUE;
}

int FeMain::EventHandler()
{
	SDL_Event event;
        char* tmp = NULL;
	// TOD: Replace this with an enumeration, etc. 
	//int key_mode = 0; // 0 - normal, 1 - Z, 2 -P

	while ( SDL_PollEvent(&event) )
	{
		if ( event.type == SDL_QUIT ) 
		{
			return 1;
	        }

		if ( event.type == SDL_KEYDOWN ) 
		{

			switch (event.key.keysym.sym) 
			{	
			case SDLK_ESCAPE:
				return 1;
				break;

			case SDLK_r:
				if(nmapScan)
					nmapScan->rotateOn = !nmapScan->rotateOn;
				break;

			case SDLK_s:
				screenShot();
				break;


			case SDLK_t:
//				nmapScan->m_Config->m_TreePorts = !nmapScan->m_Config->m_TreePorts;
//				nmapScan->m_Dirty = TRUE;
				m_Config->enableTex = !m_Config->enableTex;
				break;

			case SDLK_c:
				if(nmapScan)
				{
					nmapScan->rtri = 0;
					nmapScan->x_Rot = 0;
					nmapScan->y_Rot = 0;
					nmapScan->y_Trans = 0.6f;
					nmapScan->z_Trans = -2.0f;
				}

				break;

			case SDLK_l:
				m_Config->enableLight = !m_Config->enableLight;	
				break; 
			case SDLK_a:

				tmp = openFile();
				if(tmp)
				{
					m_ScanParser->parseFile(tmp);
					delete[] tmp; // ?
					nmapScan->m_Dirty = TRUE;
				}

				break;
			case SDLK_o:

				tmp = openFile();
				if(tmp)
				{
					if(nmapScan)
					{
						delete nmapScan;
						nmapScan = NULL;
					} // if(nmapScan)

					nmapScan = new ScanDraw();
					nmapScan->m_Config = m_Config;
					nmapScan->m_TexStore = m_Config->getTextureStore();
					nmapScan->fpsOn = TRUE;
					nmapScan->rotateOn = FALSE;			

					m_ScanParser->setScan(nmapScan);
					m_ScanParser->parseFile(tmp);

					delete[] tmp;
				}

				break;

			case SDLK_UP:
				nmapScan->x_Rot += 1.5f;
				break;
			case SDLK_DOWN:
				nmapScan->x_Rot -= 1.5f;
				break;
			case SDLK_LEFT:
				nmapScan->y_Rot -= 1.5f;
				break;
			case SDLK_RIGHT:
				nmapScan->y_Rot += 1.5f;
				break;
			case SDLK_MINUS:
				nmapScan->rotSpeed -= 0.1f;
				break;
			case SDLK_PLUS:
			case SDLK_EQUALS:
				nmapScan->rotSpeed += 0.1f;
				break;

			case SDLK_RSHIFT:
			case SDLK_LSHIFT:
				key_mode = 1;	
				break;
			
			case SDLK_RCTRL:
			case SDLK_LCTRL:
				key_mode = 2;	
				break;


			default:
				return 0;
			}

		} // if ( event.type == SDL_KEYDOWN )

		if(event.type == SDL_KEYUP)
		{
			key_mode = 0;
		}

		if ( event.type == SDL_MOUSEMOTION)
		{
			if(mouseDown)
			{

				// Pan Up/Down
				if(key_mode == 2)
				{
					nmapScan->y_Trans += 0.005f*(float)(prevY - event.motion.y);
				}
				else if(key_mode == 1)
				{ // Zoom In/Out
					nmapScan->z_Trans -= 0.005f*(float)(prevX - event.motion.x);
				}
				else	// Rotate around
				{
					nmapScan->y_Rot -= 0.5f*(float)(prevX - event.motion.x);
				}

				prevY = event.motion.y;
				prevX = event.motion.x;

			} // if(mouseDown)
		}
		if ( event.type == SDL_MOUSEBUTTONDOWN ) 
		{
			if(event.button.button == SDL_BUTTON_LEFT)
			{
				prevX = event.motion.x;
				prevY = event.motion.y;
				mouseDown = TRUE;
			}
			else
			{
				glRasterPos2i(0, 0);

			}
//			printf("x: %d, y: %d \n", event.motion.x,  event.motion.y);
		} // if ( event.type == SDL_MOUSEBUTTONDOWN )

		if ( event.type == SDL_MOUSEBUTTONUP ) 
		{
			if(event.button.button == SDL_BUTTON_LEFT)
			{
				mouseDown = FALSE;
			}
			//printf("x: %d, y: %d \n", event.motion.x,  event.motion.y);
		}
	} // while ( SDL_PollEvent(&event) )
	return 0;
}
int FeMain::InitGL(int Width, int Height)
{
    glViewport(0, 0, Width, Height);

	if(m_Config->enableEarth)
		if (!LoadGLTextures())
			return FALSE;

	glShadeModel(GL_SMOOTH);
	glClearColor(0.0f, 0.0f, 0.0f, 0.5f);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, white_light);
	glLightfv(GL_LIGHT0, GL_SPECULAR, white_light);

	glLightfv(GL_LIGHT1, GL_POSITION, light_position2);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, white_light);
	glLightfv(GL_LIGHT1, GL_SPECULAR, white_light);

	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHT1);
	
	glClearDepth(1.0f);										// Depth Buffer Setup
	glEnable(GL_DEPTH_TEST);
//	glCullFace(GL_BACK);
//	glEnable(GL_CULL_FACE);

	glDepthFunc(GL_LESS);

	// We might actually not need these
//	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);		// Really Nice Perspective Calculations
//	glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);					// Set Line Antialiasing
    glMatrixMode(GL_PROJECTION);

	glLoadIdentity();
    gluPerspective(45.0f,(GLfloat)Width/(GLfloat)Height,0.1f,100.0f);	// Calculate The Aspect Ratio Of The Window
    glMatrixMode(GL_MODELVIEW);

	glLoadIdentity();

	// Test Extensions
	testCheckExt();


	return TRUE;										// Initialization Went OK
}

int FeMain::DrawGLScene( )
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	glBindTexture(GL_TEXTURE_2D, texture[0]);
	
	if(nmapScan)
		nmapScan->draw();

	drawFps(nmapScan->fpsOn);

//	drawMenu();

	++ff;
	SDL_GL_SwapBuffers();

	if(glGetError() != GL_NO_ERROR)
	{
		cout<<"glGetError returned an error"<<endl;
		cout<< gluErrorString(glGetError())<<endl;
	}


#if defined(WIN32) && defined(MEM_DEBUG)
	_CrtMemCheckpoint( &s1 );
#endif


	return TRUE;
}

int FeMain::LoadGLTextures(void)
{	
    // Load Texture
    SDL_Surface *image1;
    
	image1 = m_glUtil->LoadBMP(m_Config->m_RootTex.c_str(), TRUE, FALSE);
    
    if (!image1) {
        SDL_Quit();
        exit(1);
    }

    // Create Texture	
    glGenTextures(1, &texture[0]);
    glBindTexture(GL_TEXTURE_2D, texture[0]);   // 2d texture (x and y size)

    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); // scale linearly when image bigger than texture
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); // scale linearly when image smalled than texture

    // 2d texture, level of detail 0 (normal), 3 components (red, green, blue), x size from image, y size from image, 
    // border 0 (normal), rgb color data, unsigned byte data, and finally the data itself.
    glTexImage2D(GL_TEXTURE_2D, 0, 3, image1->w, image1->h, 0, GL_RGB, GL_UNSIGNED_BYTE, image1->pixels);

	SDL_FreeSurface(image1);
	return TRUE;
};

int FeMain::screenShot()
{
	PNGScreen* pngs = new PNGScreen();
	pngs->ss_id = ss_id++;
	pngs->init();
	pngs->initText();
	pngs->doStuff();
	pngs->kill();
	return 0;
}

char* FeMain::openFile()
{

#ifdef _WIN32
	char* szFile2 = new char[1024];
	char szFile[1024];

	OPENFILENAME file;
	// Initialize OPENFILENAME
	ZeroMemory(&file, sizeof(file));
	file.lStructSize = sizeof(file);
	file.hwndOwner = NULL;
	file.lpstrFile = szFile;

	file.lStructSize = sizeof (OPENFILENAME);
	file.hwndOwner = NULL;
	file.lpstrFilter = "*.xml\0";
	file.lpstrInitialDir = "\\\0";
	file.lpstrFile[0] = '\0';
	file.nMaxFile = sizeof(szFile);
	file.nFilterIndex = 1;
	file.lpstrFileTitle = NULL;
	file.nMaxFileTitle = 0;
	file.lpstrInitialDir = NULL;
	file.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

	if(!GetOpenFileName(&file))
	{
		return NULL;

	} // if( GetOpenFileName(&file))

	strcpy(szFile2, szFile);
	return szFile2;

#else
	return NULL;
#endif
}

void FeMain::drawFps(bool screenfps)
{
	thisframe=SDL_GetTicks(); 
	if((thisframe-lastframe) > 1000) 
	{
		fps=ff; 
		ff=0;
		lastframe=thisframe;
	} 

	rFps=fps;
}

void drawMenu()
{
// Lets try to draw some things here

	Uint32 rmask, gmask, bmask, amask;
	#if SDL_BYTEORDER == SDL_BIG_ENDIAN
    rmask = 0xff000000;
    gmask = 0x00ff0000;
    bmask = 0x0000ff00;
    amask = 0x000000ff;
	#else
    rmask = 0x000000ff;
    gmask = 0x0000ff00;
    bmask = 0x00ff0000;
    amask = 0xff000000;
	#endif
	
	SDL_Surface* image1 = SDL_CreateRGBSurface(SDL_SWSURFACE, 32, 32, 32,rmask, gmask, bmask, amask);
	SDL_FillRect(image1, NULL, 0xFFFFFF);

	glDrawPixels(image1->w, image1->h, GL_RGBA,GL_UNSIGNED_BYTE, image1->pixels);

// end of drawing stuff.
}

/**
	Verifies that enabled extensions, are actualy present
*/
void FeMain::testCheckExt()
{
	//check whether extension is supported
	if (strstr((char*)glGetString(GL_EXTENSIONS), "GL_ARB_vertex_buffer_object"))
	{
		//get function pointers using SDL_GL_GetProcAddress(..)
#ifdef WIN32		// (could be using wglGetProcAddress(..) too)
		pglBindBufferARB = (PFNGLBINDBUFFERARBPROC)SDL_GL_GetProcAddress("glBindBufferARB");
		pglGenBuffersARB = (PFNGLGENBUFFERSARBPROC)SDL_GL_GetProcAddress("glGenBuffersARB");
		pglBufferDataARB = (PFNGLBUFFERDATAARBPROC)SDL_GL_GetProcAddress("glBufferDataARB");
#endif
//		m_Config->enableVBO = true;
	}
	else
	{
		m_Config->enableVBO = false;
	}

	if (strstr((char*)glGetString(GL_EXTENSIONS), "GL_EXT_compiled_vertex_array"))
	{
#ifdef WIN32 
		glLockArraysEXT = (PFNGLLOCKARRAYSEXTPROC) SDL_GL_GetProcAddress("glLockArraysEXT");
		glUnlockArraysEXT = (PFNGLUNLOCKARRAYSEXTPROC) SDL_GL_GetProcAddress("glUnlockArraysEXT");
#endif
	}
	else
	{
		m_Config->enableCVA = false;
	}

}


Uint32 timerProc(Uint32 interval, void *param)
{
	char title[1024];
  	sprintf(title, "fe3d v %s build %i - %ifps", PRODUCT_VERSION, BUILD_NUM, rFps);

#if !defined(__APPLE__) && !defined(__MACH__)
	SDL_WM_SetCaption(title, NULL);
#endif
	return interval;
}
