/**************************************************************************
 *	Copyright (C) 2004  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#if _MSC_VER > 1000
#pragma once
#endif

#ifndef _H_F3_HOST_DRAW
#define _H_F3_HOST_DRAW

#include "fe_draw.hpp"

typedef char STATE;
typedef char PROTOCOL;

class PortDraw : public FeDraw
{
public:
	PortDraw(int port, STATE state, PROTOCOL prot);
	~PortDraw();

public:
	void draw(GLenum mode = GL_RENDER);
	void generate(void);
	void drawRootNode(char* ip);

public: // Setters and Getters
	void setService(char* service);
	int getPort()	{ return m_Port; };
	STATE getState()	{ return m_State; };

private:
	GLuint	portTexture;
	GLuint	serviceTexture;
	char*	m_Service;
	bool	m_Dirty;
	GLuint	m_dList;
	int		m_Port;
	STATE	m_State;
	PROTOCOL m_Protocol;

public:
	static const STATE STATE_CLOSED	= 1;
	static const STATE STATE_OPEN		= 2;
	static const STATE STATE_FILTERED = 3;
	static const PROTOCOL PROT_TCP = 0;
	static const PROTOCOL PROT_UDP = 1;
};

	// Other;

class HostDraw : public FeDraw
{
public:
	HostDraw();
	~HostDraw();

	void generate(void);

	void draw(GLenum mode = GL_RENDER);
	void drawIP();
	void drawTower();
	void drawPorts();
	void drawHostGeneric();
	void drawHostRouter();

	void addName(string* host);
	void addPort(PortDraw* port);

public: // Setters and Getters
	void setIp(const char* ip);
	string getIp() { return m_ipAddress; }
	void setType(char* type) {m_HostType = type; };
	void setVendor(char* vendor) { m_Vendor = vendor; };
	void setOSFam(OS_FAMILY::OS f) { m_OSFamily = f; };
	OS_FAMILY::OS getOSFam() { return m_OSFamily; };
	
	int getPortCount();
	
private:
	float m_Radius;
	GLuint texOs;
	GLuint texFlag;
	GLuint texHost;
	GLuint m_dList;
	GLuint texIp;
	GLfloat* m_vaLines;

	bool m_Dirty;  // regenerate scene?
	
	string m_ipAddress;
	string m_HostType;
	string m_Vendor;

	vector<PortDraw*> m_Ports;
	vector<string*> m_Hostnames;

	OS_FAMILY::OS	m_OSFamily;

};

#endif // _H_F3_HOST_DRAW
